import { Injectable, inject } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { AuthState, UserInfo } from '../interface/auth.interface';
import { StorageService } from './storage.service';

@Injectable({ providedIn: 'root' })
export class AuthStateService {
  private readonly storageService = inject(StorageService);

  private authStateSubject = new BehaviorSubject<AuthState>({
    isAuthenticated: false,
    user: null,
    token: null
  });

  authState$ = this.authStateSubject.asObservable();

  // State getters
  getCurrentState = (): AuthState => this.authStateSubject.value;
  isAuthenticated = (): boolean => this.getCurrentState().isAuthenticated;
  getCurrentUser = (): UserInfo | null => this.getCurrentState().user;
  getCurrentToken = (): string | null => this.getCurrentState().token;

  // Role checks
  hasRole = (role: string): boolean => this.getCurrentUser()?.role === role;
  isAdmin = (): boolean => this.hasRole('admin');
  isCustomer = (): boolean => this.hasRole('customer');

  // State updates
  setAuthenticatedState = (user: UserInfo, token: string): void => {
    this.authStateSubject.next({
      isAuthenticated: true,
      user,
      token
    });
  };

  setUnauthenticatedState = (): void => {
    this.authStateSubject.next({
      isAuthenticated: false,
      user: null,
      token: null
    });
  };

  // Initialize state from storage
  loadStateFromStorage = (): void => {
    if (this.storageService.hasValidAuthData()) {
      const user = this.storageService.getUser();
      const token = this.storageService.getToken();
      
      if (user && token) {
        this.setAuthenticatedState(user, token);
        return;
      }
    }
    
    this.setUnauthenticatedState();
  };

  // Complete logout
  logout = (): void => {
    this.storageService.clearAllAuth();
    this.setUnauthenticatedState();
  };
}
