import { Injectable, inject } from '@angular/core';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Observable, throwError, of } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { 
  Job, 
  JobListResponse, 
  JobResponse, 
  CreateJobRequest, 
  UpdateJobRequest, 
  JobStats, 
  JobFilter, 
  JobPagination,
  JobLocation,
  JobType 
} from '../interface/job.interface';
import { environment } from '../../../../environments/environment';
import { StorageService } from './storage.service';

// API response format that matches your data structure
interface ApiJobResponse {
  message: string;
  responseData: {
    items: ApiJob[];
  };
}

interface ApiJob {
  id: string;
  type: string;
  created_at: string;
  company: string;
  company_url: string;
  location: string;
  title: string;
  description: string;
}

@Injectable({ providedIn: 'root' })
export class JobApiService {
  private readonly http = inject(HttpClient);
  private readonly storageService = inject(StorageService);
  private readonly API_URL = `${environment.API_DOMAIN}/jobs`;

  // HTTP Headers
  private getHeaders = (): HttpHeaders => {
    const headers: { [key: string]: string } = { 'Content-Type': 'application/json' };
    const token = this.storageService.getToken();
    if (token) {
      headers['Authorization'] = `Bearer ${token}`;
    }
    return new HttpHeaders(headers);
  };

  private buildParams = (filter?: JobFilter, pagination?: JobPagination): HttpParams => {
    let params = new HttpParams();
    
    if (pagination) {
      params = params.set('page', pagination.page.toString());
      params = params.set('limit', pagination.limit.toString());
      if (pagination.sortBy) params = params.set('sortBy', pagination.sortBy);
      if (pagination.sortOrder) params = params.set('sortOrder', pagination.sortOrder);
    }

    if (filter) {
      if (filter.type) params = params.set('type', filter.type);
      if (filter.company) params = params.set('company', filter.company);
      if (filter.location) params = params.set('location', filter.location);
      if (filter.search) params = params.set('search', filter.search);
    }

    return params;
  };

  private convertApiJobToJob = (apiJob: ApiJob): Job => {
    return {
      id: apiJob.id,
      title: apiJob.title,
      description: apiJob.description,
      company: apiJob.company,
      company_url: apiJob.company_url,
      location: apiJob.location, 
      type: this.mapApiTypeToJobType(apiJob.type),
      created_at: apiJob.created_at,
      how_to_apply: "", 
      company_logo: null 
    };
  };

  private mapApiTypeToJobType = (apiType: string): JobType => {
    if (apiType.toLowerCase().includes('part')) return 'Part Time';
    if (apiType.toLowerCase().includes('remote')) return 'Remote';
    return 'Full Time';
  };

  // API Calls
  getJobs = (filter?: JobFilter, pagination?: JobPagination): Observable<JobListResponse> => {
    const params = this.buildParams(filter, pagination);
    
    return this.http.get<ApiJobResponse>(this.API_URL, {
      headers: this.getHeaders(),
      params
    }).pipe(
      map(response => {
        const jobs = response.responseData.items.map(this.convertApiJobToJob);
        return {
          message: response.message,
          responseData: {
            jobs: jobs,
            total: jobs.length,
            page: pagination?.page || 1,
            limit: pagination?.limit || 10
          },
          success: true,
          status: 200
        };
      }),
      catchError(this.handleError)
    );
  };



  createJob = (request: CreateJobRequest): Observable<JobResponse> => {
    const apiJobData = {
      title: request.job.title,
      description: request.job.description,
      company: request.job.company,
      company_url: request.job.company_url,
      location: request.job.location,
      type: request.job.type
    };

    return this.http.post<ApiJob>(this.API_URL, apiJobData, {
      headers: this.getHeaders()
    }).pipe(
      map(apiJob => ({
        message: 'Job created successfully',
        responseData: {
          job: this.convertApiJobToJob(apiJob)
        },
        success: true,
        status: 201
      })),
      catchError(this.handleError)
    );
  };

  updateJob = (request: UpdateJobRequest): Observable<JobResponse> => {
    // Convert internal job format to API format
    const apiJobData = {
      title: request.job.title,
      description: request.job.description,
      company: request.job.company,
      company_url: request.job.company_url,
      location: request.job.location,
      type: request.job.type
    };

    return this.http.put<ApiJob>(`${this.API_URL}/${request.id}`, apiJobData, {
      headers: this.getHeaders()
    }).pipe(
      map(apiJob => ({
        message: 'Job updated successfully',
        responseData: {
          job: this.convertApiJobToJob(apiJob)
        },
        success: true,
        status: 200
      })),
      catchError(this.handleError)
    );
  };

  deleteJob = (id: string): Observable<{ success: boolean; message: string }> =>
    this.http.delete(`${this.API_URL}/${id}`, {
      headers: this.getHeaders()
    }).pipe(
      map(() => ({
        success: true,
        message: 'Job deleted successfully'
      })),
      catchError(this.handleError)
    );

  getJobStats = (): Observable<{ success: boolean; data: JobStats }> => {
    // Since the API doesn't provide stats endpoint, we'll calculate from jobs
    return this.getJobs().pipe(
      map(response => {
        const jobs = response.responseData.jobs;
        const stats: JobStats = {
          totalJobs: jobs.length,
          companiesCount: new Set(jobs.map(job => job.company)).size
        };
        return {
          success: true,
          data: stats
        };
      }),
      catchError(error => {
        console.error('Failed to get job stats:', error);
        return of({
          success: false,
          data: {
            totalJobs: 0,
            companiesCount: 0
          }
        });
      })
    );
  };

  // Error handling
  private handleError = (error: any): Observable<never> => {
    let message = 'An error occurred';
    
    if (error.error?.message) {
      message = error.error.message;
    } else if (error.message) {
      message = error.message;
    }

    const statusMessage = this.getStatusMessage(error.status, message);
    return throwError(() => ({ status: error.status, message: statusMessage }));
  };

  private getStatusMessage = (status: number, defaultMessage: string): string => {
    const statusMessages: Record<number, string> = {
      400: 'Invalid request data',
      401: 'Authentication required',
      403: 'Access denied',
      404: 'Job not found',
      409: 'Job already exists',
      422: 'Validation failed',
      500: 'Server error. Please try again later.',
      503: 'Service unavailable'
    };
    
    return statusMessages[status] || defaultMessage;
  };
}
